@@ -342,8 +342,29 @@ class BigQueryExecutor:
                      clustering=None,
                      priority='INTERACTIVE',
                      description=None,
+                     increment_field=None,
                      **kwargs):
-        """ create a bigquery table from a sql query """
+        """ create a bigquery table from a sql query or file
+        
+        Parameters:
+            table_id:
+            dataset_id:
+            project_id:
+            sql:
+            file:
+            location:
+            schema_path:
+            partition:
+            partition_field:
+            clustering: A list of field names that the table will be clustered on, e.g. [field1, field2]
+            priority:
+            description:
+            increment_field: The field on which the incremental load for a non-partitioned table is based. Allows for existing entries to be replaced.
+                             Requires "FROM_DATE" and "TO_DATE" env vars to be set to operate correctly.
+
+        Returns:
+            Configured query job
+        """
 
         if sql is None and file is None:
             raise BigQueryExecutorError("Either SQL or file containing the SQL must be provided")
@@ -358,13 +379,24 @@ class BigQueryExecutor:
                 partition=partition,
                 partition_field=partition_field,
                 dataset_id=dataset_id,
-                project_id=project_id
+                project_id=project_id,
+                clustering=clustering
             )
             if write_disposition == "WRITE_TRUNCATE":
                 self.truncate_table(dataset_id=dataset_id, table_id=table_id,project_id=project_id)
                 write_disposition = "WRITE_EMPTY"
-        else:
-            pass
+
+        if increment_field:
+            logging.info(f"Increment field set to {increment_field}")
+            try:
+                from_date = os.environ["FROM_DATE"]
+                to_date = os.environ["TO_DATE"]
+            except KeyError:
+                raise KeyError("Env variables FROM_DATE and TO_DATE have not been set")
+            if self.check_increment_dates(dataset_id, table_id, increment_field, from_date, to_date):
+                logging.info(f'Discovered existing records in {table_id} between {from_date} and {to_date}')
+                self.delete_backfill_increments(dataset_id, table_id, increment_field, from_date, to_date)
+                logging.info("Existing records deleted")
 
         job_config = bigquery.QueryJobConfig()
         job_config.destination = self.get_table_ref(dataset_id, table_id,project_id=project_id)
@@ -372,19 +404,20 @@ class BigQueryExecutor:
         job_config.use_legacy_sql = use_legacy_sql
         job_config.create_disposition = bigquery.CreateDisposition.CREATE_IF_NEEDED
         job_config.priority = set_priority(priority)
+        job_config.clustering_fields = clustering
+        
         if partition:
             if not partition_field or partition_field == '_PARTITIONTIME':
                 job_config.time_partitioning = bigquery.table.TimePartitioning(
                     type_='DAY'
                 )
-                job_config.clustering_fields = clustering
             elif isinstance(partition_field, str):
                 job_config.time_partitioning = bigquery.table.TimePartitioning(
                     field=partition_field
                 )
             else:
                 raise ValueError("partition_field should be a string")
-
+        
         query_job = self.client.query(
             sql,
             location=location,
@@ -404,6 +437,73 @@ class BigQueryExecutor:
 
         return query_job
 
+    def check_increment_dates(self, 
+                              dataset_id, 
+                              table_id, 
+                              increment_field, 
+                              from_date, 
+                              to_date):
+        """ Checks whether increment dates to be added are in existing table.
+
+        Parameters:
+            table_id (string): BigQuery table ID.
+            dataset_id (string): BigQuery dataset ID.
+            increment_field: Field on which table will be incrementally loaded
+            project_id (string): BigQuery project ID.
+
+        Returns:
+            True if any results returned, otherwise False
+        """
+        sql = f"""SELECT
+                    {increment_field}
+                  FROM
+                    `{dataset_id}.{table_id}`
+                  WHERE
+                    {increment_field} BETWEEN 
+                    CAST('{from_date}' AS DATE) AND CAST('{to_date}' AS DATE)
+                """
+        result = self.execute_sql(
+            sql=sql,
+        )
+        if result.shape[0] != 0:
+            return True
+        return False
+
+
+    def delete_backfill_increments(self,
+                                   dataset_id,
+                                   table_id,
+                                   increment_field,
+                                   from_date,
+                                   to_date
+                                   ):
+        """
+        Delete increments from table in backfill between from_date and to_date
+
+        Args:
+            table_id (string): table in backfill dataset that the increments will be deleted from
+            increment_field (string): the time-unit column that the table is incremented on
+            from_date (date): date of the first records to be deleted
+            to_date (date): date of the latest records to be deleted
+
+        Returns:
+            None
+        """
+        client = bigquery.Client()
+
+        sql = f"""
+               DELETE
+                 `{dataset_id}.{table_id}`
+               WHERE
+                 {increment_field} BETWEEN
+                   CAST('{from_date}' AS DATE) AND CAST('{to_date}' AS DATE)
+               """
+
+        query_job = client.query(
+            sql
+        )
+
+        query_job.result() # wait for job to complete
 
     def delete_backfill_partitions(self,
                                    backfill_dataset_id,
